@@ -1,4 +1,13 @@
-/* pam_auth.c -- PAM authentication */
+/*
+ * pam_auth.c -- PAM authentication
+ *
+ * $Id: pam_auth.c,v 1.7 1997/04/05 06:53:52 morgan Exp $
+ *
+ * $Log: pam_auth.c,v $
+ * Revision 1.7  1997/04/05 06:53:52  morgan
+ * fail-delay changes
+ *
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -11,24 +20,17 @@ int pam_authenticate(pam_handle_t *pamh, int flags)
 
     D(("pam_authenticate called"));
 
-    IF_NO_PAMH("pam_authenticate",pamh,PAM_SYSTEM_ERR);
-    retval = _pam_dispatch(pamh, flags, PAM_AUTHENTICATE);
+    _pam_sanitize(pamh);
 
-#ifdef PAM_FAIL_DELAY_ON
     _pam_start_timer(pamh);    /* we try to make the time for a failure
 				  independent of the time it takes to
 				  fail */
-#endif /* PAM_FAIL_DELAY_ON */
 
-    /*
-     * this is for security. We reset the auth-tokens here.
-     */
-    pam_set_item(pamh,PAM_AUTHTOK,NULL);
-    pam_set_item(pamh,PAM_OLDAUTHTOK,NULL);
+    IF_NO_PAMH("pam_authenticate",pamh,PAM_SYSTEM_ERR);
+    retval = _pam_dispatch(pamh, flags, PAM_AUTHENTICATE);
 
-#ifdef PAM_FAIL_DELAY_ON
+    _pam_sanitize(pamh);
     _pam_await_timer(pamh, retval);   /* if unsuccessful then wait now */
-#endif /* PAM_FAIL_DELAY_ON */
 
     D(("pam_authenticate exit"));
 
