@@ -7,9 +7,12 @@
  * This file was written from a "hint" provided by the people at SUN.
  * It is subject to change before reaching Linux-PAM-1.0 .
  *
- * $Id: pam_env.c,v 1.1 1996/12/01 03:14:13 morgan Exp $
+ * $Id: pam_env.c,v 1.2 1997/02/15 15:56:48 morgan Exp $
  *
  * $Log: pam_env.c,v $
+ * Revision 1.2  1997/02/15 15:56:48  morgan
+ * liberate pamh->env structure too!
+ *
  * Revision 1.1  1996/12/01 03:14:13  morgan
  * Initial revision
  *
@@ -106,6 +109,7 @@ void _pam_drop_env(pam_handle_t *pamh)
 	pamh->env->requested = 0;
 	pamh->env->entries = 0;
 	_pam_drop(pamh->env->list);                     /* forget */
+	_pam_drop(pamh->env);                           /* forget */
     } else {
 	D(("no environment present in pamh?"));
     }
