@@ -4,7 +4,7 @@
  * created by Marc Ewing.
  * Currently maintained by Andrew G. Morgan <morgan@parc.power.net>
  *
- * $Id: pam_handlers.c,v 1.16 1997/02/15 15:57:32 morgan Exp $
+ * $Id: pam_handlers.c,v 1.17 1997/04/05 06:55:24 morgan Exp $
  *
  */
 
@@ -40,6 +40,9 @@ static int _pam_add_handler(pam_handle_t *pamh
 
 static int _pam_parse_conf_file(pam_handle_t *pamh, FILE *f
 				, const char *known_service /* specific file */
+#ifdef PAM_READ_BOTH_CONFS
+				, int not_other
+#endif /* PAM_READ_BOTH_CONFS */
     )
 {
     char buf[BUF_SIZE];
@@ -68,7 +71,13 @@ static int _pam_parse_conf_file(pam_handle_t *pamh, FILE *f
 	    this_service = tok = _pam_StrTok(buf, " \n\t", &nexttok);
 	}
 
+#ifdef PAM_READ_BOTH_CONFS
+	if (not_other)
+	    other = 0;
+	else
+#endif /* PAM_READ_BOTH_CONFS */
 	other = !_pam_strCMP(this_service, PAM_DEFAULT_SERVICE);
+
 	if (!_pam_strCMP(this_service, pamh->service_name) || other) {
 	    /* This is a service we are looking for */
 	    D(("_pam_init_handlers: Found PAM config entry for: %s"
@@ -212,13 +221,13 @@ int _pam_init_handlers(pam_handle_t *pamh)
 	      _pam_log_error("_pam_init_handlers: PAM lockfile ("
 			     PAM_LOCK_FILE ") exists - aborting");
 	      (void) close(fd_tmp);
-#ifdef PAM_FAIL_DELAY_ON
+	      /*
+	       * to avoid swamping the system with requests
+	       */
 	      _pam_start_timer(pamh);
 	      pam_fail_delay(pamh, 5000000);
 	      _pam_await_timer(pamh, PAM_ABORT);
-#else
-	      sleep(5);     /* to avoid swamping the system with requests */
-#endif /* PAM_FAIL_DELAY_ON */
+
 	      return PAM_ABORT;
 	 }
     }
@@ -248,7 +257,11 @@ int _pam_init_handlers(pam_handle_t *pamh)
 	    f = fopen(filename, "r");
 	    if (f != NULL) {
 		/* would test magic here? */
-		retval = _pam_parse_conf_file(pamh, f, pamh->service_name);
+		retval = _pam_parse_conf_file(pamh, f, pamh->service_name
+#ifdef PAM_READ_BOTH_CONFS
+					      , 0
+#endif /* PAM_READ_BOTH_CONFS */
+		    );
 		fclose(f);
 		if (retval != PAM_SUCCESS) {
 		    _pam_log_error("_pam_init_handlers: error reading %s"
@@ -260,11 +273,19 @@ int _pam_init_handlers(pam_handle_t *pamh)
 		}
 	    } else {
 		D(("unable to open %s", filename));
+#ifdef PAM_READ_BOTH_CONFS
+		D(("checking %s", PAM_CONFIG));
+
+		if ((f = fopen(PAM_CONFIG,"r")) != NULL) {
+		    retval = _pam_parse_conf_file(pamh, f, NULL, 1);
+		    fclose(f);
+		} else
+#endif /* PAM_READ_BOTH_CONFS */
+		retval = PAM_SUCCESS;
 		/*
 		 * XXX - should we log an error? Some people want to always
 		 * use "other"
 		 */
-		retval = PAM_SUCCESS;
 	    }
 	    _pam_drop(filename);
 
@@ -276,7 +297,11 @@ int _pam_init_handlers(pam_handle_t *pamh)
 		if (f != NULL) {
 		    /* would test magic here? */
 		    retval = _pam_parse_conf_file(pamh, f
-						  , PAM_DEFAULT_SERVICE);
+						  , PAM_DEFAULT_SERVICE
+#ifdef PAM_READ_BOTH_CONFS
+						  , 0
+#endif /* PAM_READ_BOTH_CONFS */
+			);
 		    fclose(f);
 		    if (retval != PAM_SUCCESS) {
 			_pam_log_error("_pam_init_handlers: error reading %s"
@@ -302,7 +327,11 @@ int _pam_init_handlers(pam_handle_t *pamh)
 		return PAM_ABORT;
 	    }
 
-	    retval = _pam_parse_conf_file(pamh, f, NULL);
+	    retval = _pam_parse_conf_file(pamh, f, NULL
+#ifdef PAM_READ_BOTH_CONFS
+					  , 0
+#endif /* PAM_READ_BOTH_CONFS */
+		);
 
 	    D(("closing configuration file"));
 	    fclose(f);
