@@ -1,7 +1,7 @@
 /*
  * pam_private.h
  *
- * $Id: pam_private.h,v 1.11 1997/02/15 16:01:21 morgan Exp $
+ * $Id: pam_private.h,v 1.12 1997/04/05 06:57:37 morgan Exp $
  *
  * This is the Linux-PAM Library Private Header. It contains things
  * internal to the Linux-PAM library. Things not needed by either an
@@ -12,6 +12,9 @@
  * Creator: Marc Ewing.
  * 
  * $Log: pam_private.h,v $
+ * Revision 1.12  1997/04/05 06:57:37  morgan
+ * added sanitize prototype
+ *
  * Revision 1.11  1997/02/15 16:01:21  morgan
  * update version and #defines for /etc/pam.d/ code
  *
@@ -234,6 +237,8 @@ int _pam_mkargv(char *s, char ***argv, int *argc);
 
 void _pam_log_error(const char *format, ...);
 
+void _pam_sanitize(pam_handle_t *pamh);
+
 /*
  * XXX - Take care with this. It could confuse the logic of a trailing
  *       else
