@@ -1,11 +1,14 @@
 /* pam_group module */
 
 /*
- * $Id: pam_group.c,v 1.6 1997/01/04 21:57:49 morgan Exp $
+ * $Id: pam_group.c,v 1.7 1997/02/15 17:31:48 morgan Exp $
  *
  * Written by Andrew Morgan <morgan@parc.power.net> 1996/7/6
  *
  * $Log: pam_group.c,v $
+ * Revision 1.7  1997/02/15 17:31:48  morgan
+ * time parsing more robust
+ *
  * Revision 1.6  1997/01/04 21:57:49  morgan
  * fixed warning about setgroups not being defined
  *
@@ -21,7 +24,7 @@
  */
 
 const static char rcsid[] =
-"$Id: pam_group.c,v 1.6 1997/01/04 21:57:49 morgan Exp $;\n"
+"$Id: pam_group.c,v 1.7 1997/02/15 17:31:48 morgan Exp $;\n"
 "Version 0.5 for Linux-PAM\n"
 "Copyright (c) Andrew G. Morgan 1996 <morgan@parc.power.net>\n";
 
@@ -426,7 +429,7 @@ static boolean check_time(const void *AT, const char *times, int len, int rule)
      time_start = 0;
      for (i=0; len > 0 && i < 4 && isdigit(times[i+j]); ++i, --len) {
 	  time_start *= 10;
-	  time_start += times[i+j]-'0';  /* is this portable? */
+	  time_start += times[i+j]-'0';       /* is this portable? */
      }
      j += i;
 
@@ -434,13 +437,14 @@ static boolean check_time(const void *AT, const char *times, int len, int rule)
 	  time_end = 0;
 	  for (i=1; len > 0 && i < 5 && isdigit(times[i+j]); ++i, --len) {
 	       time_end *= 10;
-	       time_end += times[i+j]-'0';    /* is this portable */
+	       time_end += times[i+j]-'0';    /* is this portable? */
 	  }
 	  j += i;
      } else
 	  time_end = -1;
 
-     if (i != 5 || time_end == -1 || times[j] != '\0') {
+     D(("i=%d, time_end=%d, times[j]='%c'", i, time_end, times[j]));
+     if (i != 5 || time_end == -1) {
 	  _log_err("no/bad times specified (rule #%d)", rule);
 	  return TRUE;
      }
