@@ -1,6 +1,7 @@
 /*
  * pam_limits - impose resource limits when opening a user session
  *
+ * 1.5 - Elliot Lee's "max system logins patch"
  * 1.4 - addressed bug in configuration file parser
  * 1.3 - modified the configuration file format
  * 1.2 - added 'debug' and 'conf=' arguments
@@ -51,8 +52,11 @@ struct user_limits_struct {
 static struct user_limits_struct limits[RLIM_NLIMITS];
 static int login_limit;     /* the max logins limit */
 static int login_limit_def; /* which entry set the login limit */
+static int flag_numsyslogins; /* whether to limit logins only for a
+				 specific user or to count all logins */
 
 #define LIMIT_LOGIN RLIM_NLIMITS+1
+#define LIMIT_NUMSYSLOGINS RLIM_NLIMITS+2
 #define LIMIT_SOFT  1
 #define LIMIT_HARD  2
 
@@ -134,15 +138,20 @@ static int check_logins(const char *name, int limit, int ctrl)
 #endif
         if (ut->UT_USER[0] == '\0')
             continue;
-        if (strncmp(name, ut->UT_USER, sizeof(ut->UT_USER)) != 0)
+        if (!flag_numsyslogins
+	    && strncmp(name, ut->UT_USER, sizeof(ut->UT_USER)) != 0)
             continue;
         if (++count >= limit)
             break;
     }
     endutent();
     if (count >= limit) {
-        _pam_log(LOG_WARNING, "Too many logins (max %d) for %s\n",
-                    limit, name);
+	if (name) {
+	    _pam_log(LOG_WARNING, "Too many logins (max %d) for %s",
+		     limit, name);
+	} else {
+	    _pam_log(LOG_WARNING, "Too many system logins (max %d)", limit);
+	}
         return LOGIN_ERR;
     }
     return 0;
@@ -287,9 +296,13 @@ static void process_limit(int source, const char *lim_type,
 	limit_item = RLIMIT_MEMLOCK;
     else if (strcmp(lim_item, "as") == 0)
 	limit_item = RLIMIT_AS;
-    else if (strcmp(lim_item, "maxlogins") == 0)
+    else if (strcmp(lim_item, "maxlogins") == 0) {
 	limit_item = LIMIT_LOGIN;
-    else {
+	flag_numsyslogins = 0;
+    } else if (strcmp(lim_item, "maxsyslogins") == 0) {
+	limit_item = LIMIT_NUMSYSLOGINS;
+	flag_numsyslogins = 1;
+    } else {
         _pam_log(LOG_DEBUG,"unknown limit item '%s'", lim_item);
         return;
     }
@@ -300,7 +313,7 @@ static void process_limit(int source, const char *lim_type,
 	limit_type=LIMIT_HARD;
     else if (strcmp(lim_type,"-")==0)
 	limit_type=LIMIT_SOFT | LIMIT_HARD;
-    else if (limit_item != LIMIT_LOGIN) {
+    else if (limit_item != LIMIT_LOGIN && limit_item != LIMIT_NUMSYSLOGINS) {
         _pam_log(LOG_DEBUG,"unknown limit type '%s'", lim_type);
         return;
     }
@@ -335,7 +348,7 @@ static void process_limit(int source, const char *lim_type,
             break;
     }
     
-    if (limit_item != LIMIT_LOGIN) {
+    if (limit_item != LIMIT_LOGIN && limit_item != LIMIT_NUMSYSLOGINS) {
         if (limit_type & LIMIT_SOFT)
             if (limits[limit_item].src_soft < source)
                 return;
