@@ -1,6 +1,8 @@
 /*----------------------------------------------------------------------
  * Modified for Linux-PAM by Al Longyear <longyear@netcom.com> 96/5/5
  * Modifications, Cristian Gafton 97/2/8
+ * Modifications, Peter Allgeyer 97/3
+ * Modifications (netgroups and fixes), Nicolai Langfeldt 97/3/21
  *----------------------------------------------------------------------
  * Copyright (c) 1983, 1993, 1994
  *	The Regents of the University of California.  All rights reserved.
@@ -49,7 +51,9 @@
 #include <sys/param.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#include <netdb.h>
+#include <netdb.h>       /* This is supposed(?) to contain the following */
+int innetgr(const char *, const char *, const char *,const char *);
+
 #include <stdio.h>
 #include <errno.h>
 #include <string.h>
@@ -80,6 +84,7 @@
 #define PAM_SM_AUTH  /* only defines this management group */
 
 #include <security/pam_modules.h>
+#include <security/_pam_macros.h>
 
 /*
  * Options for this module
@@ -92,6 +97,8 @@ struct _options {
     int  opt_nowarn;
     int  opt_disallow_null_authtok;
     int  opt_silent;
+    int  opt_promiscuous;
+    int  opt_suppress;
     const char *last_error;
 };
 
@@ -120,6 +127,7 @@ static void set_option (struct _options *opts, const char *arg)
     }
 
     if (strcmp (arg, "debug") == 0) {
+	D(("debugging enabled"));
 	opts->opt_debug = 1;
 	return;
     }
@@ -129,6 +137,16 @@ static void set_option (struct _options *opts, const char *arg)
 	return;
     }
 
+    if (strcmp(arg, "promiscuous")) {
+	opts->opt_promiscuous = 1;   /* used to permit '+' in ...hosts file */
+	return;
+    }
+
+    if (strcmp(arg, "suppress")) {
+	opts->opt_suppress = 1; /* used to suppress failure warning message */
+	return;
+    }
+
     /*
      * All other options are ignored at the present time.
      */
@@ -194,45 +212,107 @@ static int pam_get_ruser(pam_handle_t *pamh, const char **ruser
 }
 
 /*
- * Returns 0 if match, 1 if no match.
+ * Returns 1 if positive match, 0 if no match, -1 if negative match.
  */
 
 static int
-__icheckhost (struct _options *opts, u_long raddr, register char *lhost)
+__icheckhost (pam_handle_t *pamh, struct _options *opts, u_long raddr
+	      , register char *lhost, const char *rhost)
 {
     struct hostent *hp;
     u_long laddr;
-    char **pp;
-    
+    int negate=1;    /* Multiply return with this to get -1 instead of 1 */
+    char **pp, *user;
+
+    /* Check nis netgroup.  We assume that pam has done all needed
+       paranoia checking before we are handed the rhost */
+    if (strncmp("+@",lhost,2) == 0)
+      return(innetgr(&lhost[2],rhost,NULL,NULL));
+
+    if (strncmp("-@",lhost,2) == 0)
+      return(-innetgr(&lhost[2],rhost,NULL,NULL));
+
+    /* -host */
+    if (strncmp("-",lhost,1) == 0) {
+	negate=-1;
+	lhost++;
+    } else if (strcmp("+",lhost) == 0) {
+        (void) pam_get_item(pamh, PAM_USER, (const void **)&user);
+        D(("user %s has a `+' host entry", user));
+	if (opts->opt_promiscuous)
+	    return (1);                     /* asking for trouble, but ok.. */
+	/* If not promiscuous: handle as negative */
+	return (-1);
+    }
+
     /* Try for raw ip address first. */
     if (isdigit(*lhost) && (long)(laddr = inet_addr(lhost)) != -1)
-	return (!! (raddr ^ laddr));
+	return (negate*(! (raddr ^ laddr)));
 
     /* Better be a hostname. */
     hp = gethostbyname(lhost);
     if (hp == NULL)
-	return (1);
+	return (0);
     
     /* Spin through ip addresses. */
     for (pp = hp->h_addr_list; *pp; ++pp)
 	if (!memcmp (&raddr, *pp, sizeof (unsigned long)))
-	    return (0);
+	    return (negate);
 
     /* No match. */
-    return (1);
+    return (0);
+}
+
+/* Returns 1 on positive match, 0 on no match, -1 on negative match */
+
+static int __icheckuser(pam_handle_t *pamh, struct _options *opts
+			, const char *luser, const char *ruser
+			, const char *rhost)
+{
+    /*
+      luser is user entry from .rhosts/hosts.equiv file
+      ruser is user id on remote host
+      rhost is the remote host name
+      */
+    char *user;
+
+    /* [-+]@netgroup */
+    if (strncmp("+@",luser,2) == 0)
+	return (innetgr(&luser[2],NULL,ruser,NULL));
+
+    if (strncmp("-@",luser,2) == 0)
+	return (-innetgr(&luser[2],NULL,ruser,NULL));
+
+    /* -user */
+    if (strncmp("-",luser,1) == 0)
+	return(-(strcmp(&luser[1],ruser) == 0));
+
+    /* + */
+    if (strcmp("+",luser) == 0) {
+	(void) pam_get_item(pamh, PAM_USER, (const void **)&user);
+	_pam_log(LOG_WARNING, "user %s has a `+' user entry", user);
+	if (opts->opt_promiscuous)
+	    return(1);
+	/* If not promiscuous we handle it as a negative match */
+	return(-1);
+    }
+
+    /* simple string match */
+    return (strcmp(ruser, luser) == 0);
 }
 
 /*
- * Returns 0 if match, 1 if not ok.
+ * Returns 0 if positive match, 1 if _not_ ok.
  */
 
 static int
-__ivaliduser (struct _options *opts, FILE *hostf, u_long raddr,
-	      const char *luser, const char *ruser)
+__ivaliduser (pam_handle_t *pamh, struct _options *opts,
+	      FILE *hostf, u_long raddr,
+	      const char *luser, const char *ruser, const char *rhost)
 {
     register const char *user;
     register char *p;
-    int ch;
+    int ch, hcheck, ucheck;
     char buf[MAXHOSTNAMELEN + 128];                       /* host + login */
 
     buf[sizeof (buf)-1] = '\0';                 	/* terminate line */
@@ -277,11 +357,28 @@ __ivaliduser (struct _options *opts, FILE *hostf, u_long raddr,
 
 	/* buf -> host(?) ; user -> username(?) */
 
-	if (__icheckhost(opts, raddr, buf) == 0) {
+	/* First check host part */
+	hcheck=__icheckhost(pamh, opts, raddr, buf, rhost);
+
+	if (hcheck<0)
+	    return(1);
+
+	if (hcheck) {
+	    /* Then check user part */
 	    if (! (*user))
-	        user = luser;
-	    if (strcmp(ruser, user) == 0)
-	        return (0);
+		user = luser;
+
+	    ucheck=__icheckuser(pamh, opts, user, ruser, rhost);
+
+	    /* Positive 'host user' match? */
+	    if (ucheck>0)
+		return(0);
+
+	    /* Negative 'host -user' match? */
+	    if (ucheck<0)
+		return(1);
+
+	    /* Neither, go on looking for match */
 	}
     }
 
@@ -299,8 +396,9 @@ __ivaliduser (struct _options *opts, FILE *hostf, u_long raddr,
  */
 
 static int
-iruserok(struct _options *opts, u_long raddr, int superuser,
-	 const char *ruser, const char *luser)
+iruserok(pam_handle_t *pamh,
+	 struct _options *opts, u_long raddr, int superuser,
+	 const char *ruser, const char *luser, const char *rhost)
 {
     const char *cp;
     struct stat sbuf;
@@ -315,7 +413,8 @@ iruserok(struct _options *opts, u_long raddr, int superuser,
 	/* try to open system hosts.equiv file */
 	hostf = fopen (_PATH_HEQUIV, "r");
 	if (hostf) {
-	    answer = __ivaliduser(opts, hostf, raddr, luser, ruser);
+	    answer = __ivaliduser(pamh, opts, hostf, raddr, luser
+				  , ruser, rhost);
 	    (void) fclose(hostf);
 	    if (answer == 0)
 		return 0;      /* remote host is equivalent to localhost */
@@ -348,22 +447,32 @@ iruserok(struct _options *opts, u_long raddr, int superuser,
 	return 1;                               /* to dangerous to try */
     }
 
-    (void)strcpy(pbuf, pwd->pw_dir);
-    (void)strcat(pbuf, USER_RHOSTS_FILE);
+    (void) strcpy(pbuf, pwd->pw_dir);
+    (void) strcat(pbuf, USER_RHOSTS_FILE);
 
     /*
-     * Change effective uid while opening .rhosts.  If root and
-     * reading an NFS mounted file system, can't read files that
-     * are protected read/write owner only.
+     * Change effective uid while _reading_ .rhosts. (not just
+     * opening.  If root and reading an NFS mounted file system,
+     * can't read files that are 0600 as .rhosts files should be.
      */
 
-    uid = geteuid();
-    (void)seteuid(pwd->pw_uid);
+    /* We are root, this will not fail */
+#ifdef linux
+    /* If we are on linux the better way is setfsuid */
+    uid = setfsuid(pwd->pw_uid);
     hostf = fopen(pbuf, "r");
-    (void)seteuid(uid);
-	
-    if (hostf == NULL)
-	return 1;
+#else
+    uid = geteuid();
+    (void) seteuid(pwd->pw_uid);
+    hostf = fopen(pbuf, "r");
+#endif
+
+    if (hostf == NULL) {
+        if (opts->opt_debug)
+	    _pam_log(LOG_DEBUG,"Could not open %s file",pbuf);
+	answer = 1;
+	goto exit_function;
+    }
 
     /*
      * If not a regular file, or is owned by someone other than
@@ -384,17 +493,33 @@ iruserok(struct _options *opts, u_long raddr, int superuser,
     /* If there were any problems, quit. */
     if (cp) {
 	opts->last_error = cp;
-	fclose(hostf);
-	return 1;
+	answer = 1;
+	goto exit_function;
     }
 
-    answer = __ivaliduser (opts, hostf, raddr, luser, ruser);
-    (void) fclose(hostf);
+    answer = __ivaliduser (pamh, opts, hostf, raddr, luser, ruser, rhost);
+
+exit_function:
+    /*
+     * Go here to exit after the fsuid/euid has been adjusted so that
+     * they are reset before we exit.
+     */
+
+#ifdef linux
+    setfsuid(uid);
+#else
+    (void)seteuid(uid);
+#endif
+
+    if (hostf != NULL)
+        (void) fclose(hostf);
+
     return answer;
 }
 
 static int
-pam_ruserok (struct _options *opts, const char *rhost, int superuser,
+pam_ruserok (pam_handle_t *pamh,
+	     struct _options *opts, const char *rhost, int superuser,
 	     const char *ruser, const char *luser)
 {
     struct hostent *hp;
@@ -411,7 +536,8 @@ pam_ruserok (struct _options *opts, const char *rhost, int superuser,
 	    memcpy (&addr, *ap, sizeof(addr));
 
 	    /* check user on remote host */
-	    if (iruserok(opts, addr, superuser, ruser, luser) == 0) {
+	    if (iruserok(pamh, opts, addr, superuser, ruser, luser, rhost)
+		== 0) {
 		answer = 0;                                /* success */
 		break;
 	    }
@@ -493,10 +619,12 @@ static int _pam_auth_rhosts (pam_handle_t *pamh,
 /*
  * Validate the account information.
  */
-	if (pam_ruserok (&opts, rhost, as_root, ruser, luser) != 0) {
-	    _pam_log(LOG_WARNING, "denied to %s@%s as %s: %s",
-		     ruser, rhost, luser, (opts.last_error==NULL) ?
-		     "access not allowed":opts.last_error);
+	if (pam_ruserok (pamh, &opts, rhost, as_root, ruser, luser) != 0) {
+	    if ( !opts.opt_suppress ) {
+		_pam_log(LOG_WARNING, "denied to %s@%s as %s: %s",
+			 ruser, rhost, luser, (opts.last_error==NULL) ?
+			 "access not allowed":opts.last_error);
+	    }
 	    retval = PAM_AUTH_ERR;
 	} else {
 	    _pam_log(LOG_NOTICE, "allowed to %s@%s as %s",
@@ -552,6 +680,9 @@ struct pam_module _pam_rhosts_auth_modstruct = {
 
 /*
  * $Log: pam_rhosts_auth.c,v $
+ * Revision 1.11  1997/04/05 06:26:39  morgan
+ * fairly major fixes and enhancements (see CHANGELOG for 0.57 release)
+ *
  * Revision 1.10  1997/02/09 02:09:30  morgan
  * - implementation of 'debug' argument (Cristian Gafton)
  * - we check for uid=0 accounts instead of hardcoded 'root' (Cristian Gafton)
