@@ -135,18 +135,16 @@ int pam_sm_chauthtok(	pam_handle_t *pamh,
 
 static void _pam_log(int err, const char *format, ...)
 {
-    char buf[1024];
     va_list args;
 
     va_start(args, format);
-    vsprintf(buf, format, args);
-
-    openlog("pam_unix_passwd", LOG_CONS|LOG_PID, LOG_AUTH);
-    syslog(err, buf);
+    openlog("PAM-unix_passwd", LOG_CONS|LOG_PID, LOG_AUTH);
+    vsyslog(err, format, args);
+    va_end(args);
     closelog();
 }
 
-#ifdef USE_LCKPWDF
+#ifdef NEED_LCKPWDF
 /* This is a hack, but until libc and glibc both include this function
  * by default (libc only includes it if nys is not being used, at the
  * moment, and glibc doesn't appear to have it at all) we need to have
